from datetime import datetime
from enum import IntEnum, auto

from discord.ext import commands
from tinydb import TinyDB, Query, where


class GameMode(IntEnum):
    DEFAULT = auto()
    RT = auto()
    DND = auto()
    AW = auto()


MODE_META = {GameMode.DEFAULT: 'Default die is 1d6',
             GameMode.RT: 'Default die is 1d100; reports DoS/DoF',
             GameMode.DND: 'Default die is 1d20',
             GameMode.AW: 'Default die is 1d6'}


MODE_DICE = {GameMode.DEFAULT: '1d6',
             GameMode.RT: '1d100',
             GameMode.DND: '1d20',
             GameMode.AW: '1d6'}


class Database:

    def __init__(self, path):
        self.db = TinyDB(path)
        self.rolls = self.db.table('rolls')
        self.modes = self.db.table('modes')
        self.vars = self.db.table('vars')

    def add_guilds(self, guilds):
        for guild in guilds:
            if not self.modes.get(where('guild_id') == guild.id):
                self.modes.insert({'guild_id': guild.id,
                                   'mode': GameMode.DEFAULT})

    def add_roll(self, guild, member, roll, tag, result):
        self.rolls.insert({'time': datetime.now().timestamp(),
                           'guild_id': guild.id,
                           'member_id': member.id,
                           'member_nick': member.nick,
                           'member_name': member.name,
                           'roll': roll,
                           'tag': tag,
                           'result': result})

    def query_guild_rolls(self, guild):
        RollsQ = Query()
        return self.rolls.search(RollsQ.guild_id == guild.id)

    def set_guild_mode(self, guild, mode):
        if not isinstance(mode, GameMode):
            try:
                mode = GameMode[mode]
            except KeyError:
                raise ValueError(f'{mode} is not a valid GameMode')
        
        self.modes.upsert({'guild_id': guild.id, 'mode': mode},
                          where('guild_id') == guild.id)

    def get_guild_mode(self, guild):
        result = self.modes.search(where('guild_id') == guild.id)
        if result:
            return GameMode(result[0]['mode'])
        else:
            self.set_guild_mode(guild, GameMode.DEFAULT)
            return GameMode.DEFAULT

    def set_var(self, guild, var, val):
        self.vars.upsert({'guild_id': guild.id,
                          'var': var,
                          'val': val},
                         (where('guild_id') == guild.id) & (where('var') == var))

    def get_var(self, guild, var):
        result =  self.vars.get((where('guild_id') == guild.id) & \
                                (where('var') == var))
        if result:
            return result['val']
        else:
            return None

    def del_var(self, guild, var):
        self.vars.remove((where('guild_id') == guild.id) & (where('var') == var))

    def get_guild_vars(self, guild):
        result = self.vars.search(where('guild_id') == guild.id)
        variables = {row['var']: row['val'] for row in result}
        return variables




import dice

import discord
from discord.ext import commands

import argparse
import json
import os
from pathlib import Path
import sys
import textwrap

from .logging import setup as setup_logger

from . import __version__, __splash__, __about__, __testing__
from .database import DatabaseCache
from .utils import default_log_file, default_database_dir
from .zcrit import CritCommands
from .zhistory import HistoryCommands
from .zmode import ModeCommands
from .zroll import RollCommands
from .zvars import VarCommands


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
    pass


def main():

    parser = argparse.ArgumentParser(
        description=f'{__splash__}\n{__about__}',
        formatter_class=CustomFormatter
    )
    parser.add_argument(
        '--secret-token',
        help='The secret token for the bot, '\
             'from the discord developer portal. '\
             'If you have set $ZARDOZ_TOKEN, this '\
             'option will override that.'
    )
    parser.add_argument(
        '--database-dir',
        type=lambda p: Path(p).absolute(),
        default=default_database_dir(debug=__testing__),
        help='Directory for the bot databases. Default follows the '\
             'XDG specifiction.'
    )
    parser.add_argument(
        '--log',
        type=lambda p: Path(p).absolute(),
        default=default_log_file(debug=__testing__),
        help='Path to the log file. Default follows the '\
             'XDG specifiction.'
    )
    args = parser.parse_args()

    log = setup_logger(args.log)

    TOKEN = args.secret_token
    if not TOKEN:
        try:
            TOKEN = os.environ['ZARDOZ_TOKEN']
        except KeyError:
            log.error('Must set ZARDOZ_TOKEN or use --secret-token')
            sys.exit(1)
        else:
            log.info('Got secret token from $ZARDOZ_TOKEN.')

    # get a handle for the history database
    DB = DatabaseCache(args.database_dir)

    class ZardozBot(commands.Bot):

        async def close(self):
            log.info('DatabaseCache close()')
            await DB.close()
            log.info('Bot close()')
            await super().close()
            log.info('Bot closed.')

    bot = ZardozBot(command_prefix='/')

    @bot.event
    async def on_ready():
        log.info(f'Ready: member of {bot.guilds}')
        log.info(f'Users: {bot.users}')

    @bot.command(name='zabout', help='Project info.')
    async def zabout(ctx):
        msg = f'version: {__version__}\n'\
              f'source: https://github.com/camillescott/zardoz\n'\
              f'active installs: {len(bot.guilds)}'
        await ctx.message.reply(msg)

    bot.add_cog(RollCommands(bot, DB))
    bot.add_cog(CritCommands(bot, DB))
    bot.add_cog(VarCommands(bot, DB))
    bot.add_cog(ModeCommands(bot, DB))
    bot.add_cog(HistoryCommands(bot, DB))

    bot.run(TOKEN)


if __name__ == '__main__':
    main()



import typing

from discord.ext import commands

from .database import fetch_guild_db
from .logging import LoggingMixin
from .state import GameMode, MODE_META


class ModeConvert(commands.Converter):

    async def convert(self, ctx, argument):
        try:
            converted = GameMode[argument]
        except KeyError:
            raise commands.BadArgument(f'{argument} is not a valid mode.')
        return converted


class ModeCommands(commands.Cog, LoggingMixin):

    def __init__(self, bot, db):
        self.bot = bot
        self.db = db

        super().__init__()

    @commands.group(name='zmode', help='Manage game modes for the server')
    async def zmode(self, ctx):
        if ctx.invoked_subcommand is None:
            pass
    
    @zmode.command(name='set', help='Set the mode for the server.')
    @fetch_guild_db
    async def zmode_set(self, ctx, mode: typing.Optional[ModeConvert]):
        if mode is None:
            mode = GameMode.DEFAULT
        await ctx.guild_db.set_guild_mode(mode)
        await ctx.send(f'**Set Mode:** {GameMode(mode).name}')

    @zmode.command(name='get', help='Display the server mode.')
    @fetch_guild_db
    async def zardoz_mode_get(self, ctx):
        current_mode = await ctx.guild_db.get_guild_mode()
        await ctx.send(f'**Mode:**: {GameMode(current_mode).name}\n'\
                       f'*{MODE_META[current_mode]}*')

    @zmode.command(name='list', help='List available modes.')
    async def zardoz_mode_list(self, ctx):
        modes = '\n'.join((f'{mode.name}: {MODE_META[mode]}' for mode in GameMode))
        await ctx.send(modes)




import glob
import os
import typing

import dice
import yaml

from discord.ext import commands

from .logging import LoggingMixin
from .utils import __pkg_dir__

class CritCommands(commands.Cog, LoggingMixin):

    def __init__(self, bot, db):
        self.bot = bot
        self.db = db
        super().__init__()

        self.tables = load_crit_tables(self.log)
        self.log.info(f'Loaded crits: {self.tables}')

    @commands.group(name='zcrit', help='Roll on a crit table.')
    async def zcrit(self, ctx):
        pass

    @zcrit.command(name='r')
    async def zcrit_roll(self, ctx, table_name: str, val: typing.Optional[int]):
        try:
            table = self.tables[table_name]
        except KeyError:
            self.log.error(f'Error: no such crit: {table_name}.')
            await ctx.message.reply(f'No crit table named {table_name}.')
        else:
            try:
                if val is None:
                    val, name, effect = table.roll()
                else:
                    name, effect = table.get(val)
            except ValueError:
                await ctx.message.reply(f'Bad crit table value. Perils be upon ye.')
            else:
                msg = f'**Result:** {table.die} ⤳ {val}\n'\
                      f'**Table:** {table.full_name} ({table.game}, {table.book})\n'\
                      f'**Name:** {name}\n'\
                      f'**Effect:** {effect}'
                await ctx.message.reply(msg)


def load_crit_tables(log=None):
    
    files = glob.glob(os.path.join(__pkg_dir__, 'crits', '*.yaml'))
    tables = {}
    for file_path in files:
        table = CritTable(file_path)
        tables[table.slug] = table
    return tables


class CritTable:

    def __init__(self, table_yaml):
        with open(table_yaml) as fp:
            data = yaml.safe_load(fp)

        self.full_name = data['full_name']
        self.slug = data['slug']
        self.game = data['game']
        self.book = data['book']
        self.die = data['die']
        self.rolls = data['rolls']
    
    
    def get(self, roll):
        for option in self.rolls:
            lower, upper = option['range']
            name = '' if not option['name'] else option['name'].strip()
            if lower <= roll <= upper:
                return name, ' '.join(option['effect'].split())

        raise ValueError(f'No entry for {roll}')

    def roll(self, modifers = None):
        rolled_val = int(dice.roll(self.die))
        name, effect = self.get(rolled_val)

        return rolled_val, name, effect





from datetime import datetime
import functools
import os
from pathlib import Path

import discord
from xdg import xdg_data_home


__pkg_dir__ = os.path.abspath(os.path.dirname(__file__))
__time_format__ = '%a %b %d %I:%M%p %Z'


def default_database_dir(debug=False):
    if not debug:
        return xdg_data_home().joinpath('zardoz-bot', 'databases')
    else:
        return xdg_data_home().joinpath('zardoz-bot', 'debug', 'databases')


def default_log_file(debug=False):
    if not debug:
        return xdg_data_home().joinpath('zardoz-bot', 'bot.log')
    else:
        return xdg_data_home().joinpath('zardoz-bot', 'debug', 'bot.log')


def handle_http_exception(func):

    @functools.wraps(func)
    async def wrapper(self, ctx, *args, **kwargs):
        try:
            return await func(self, ctx, *args, **kwargs)
        except discord.HTTPException:
            await ctx.message.reply('Hey broh, that\'s a big response.'\
                                    ' Try dialing back your request,'\
                                    ' I\'m only human.')
            raise
    return wrapper
